# Comparing ray-matrix and FD seismograms in 1-D model RM
# =======================================================
# Input files required:
 #chk.pl: "crt/" "writall.dat"
 #chk.pl: "crt/" "sourcex.dat"

# Data specifying the model by means of the MODEL package
  MODEL='rm-mod.dat'

# Source and receivers
  SRC='rm-src.dat'  REC='rm-rec.dat'  SOURCE='sourcex.dat'

# Stacks of fine layers at the receiver sites (program GREENSS)
  RMATRIX='rm-rm.dat'

# Source time function
  KSIG=3   SIGW=2  # Kuepper signal with 2 local maxima
  SIGF=20.         # reference frequency
  SIGA=1.          # maximum amplitude
  DER=0.5          # halfth derivative for the point source

# Data to control seismogram plotting (program SP)
  KODESP=1  SPCHRH=0.30
  SPTMIN=0.050  SPTMAX=0.280  SPTLEN=23.0  SPTDIV=4.6 SPTSUB=5
  SPXMIN=-.200  SPXMAX=0.200  SPXLEN=14.0  SPXDIV=4   SPXSUB=1
  NORMSP=1  SPAMP=-4.00  # amplitude scaling for point source in 3-D

# Hypocentral distance where the point and line sources have comparable
# amplitudes (for amplitude power scaling roughly converting line
# source to point source)
  SPDIST=0.049736  # SPDIST=1/(2*pi*VP), VP=3.2

# Ray-matrix synthetic seismograms (point source)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Data describing the filtration of the source time function
  FMIN=0.  FLOW=2.  FHIGH=110.  FMAX=120. # (cosine filter)

# Time step and time interval for the Fast Fourier Transform
  DT=0.00035  NFFT=2048

# GSE file with synthetic seismograms
  SS='rm-ss.gse'

# Data to control optional plotting of program SS
  SIGPLOT='rm-sig.ps'  MPTS=400

# Programs to execute
  crt:     'rm-crt.dat' /     # two-point ray tracing
                              # (do not link 'crt.for' with 'scropc.for'!)
  green:   'rm-fd.h' /        # Green function
  greenss: 'rm-fd.h' /        # response function
  ss:      'rm-fd.h' /        # synthetic seismograms

# FD synthetic seismograms (line source)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Grid dimensions, FD time steps, snapshots
  N1=601     N3=344     NTFD=1601     N4=80
  D1=0.001   D3=0.001   DTFD=0.000175 D4=0.0035
  O1=-.300   O3=0.000                 O4=0.000

# GSE file with synthetic seismograms
  SS='rm-fd.gse'

# Form of files with effective elastic parameters and snapshots
  FORM='formatted'

# Files with snapshots (not generated by default)
 #SNAP1='rm-fd1.out'
 #SNAP3='rm-fd3.out'

# Files with effective elastic parameters
  A11='a11.out'
  B11='b11.out'
  C11='c11.out'
  A13='a13.out'
  B13='b13.out'
  C13='c13.out'
  A31='a31.out'
  B31='b31.out'
  C31='c31.out'
  A33='a33.out'
  B33='b33.out'
  C33='c33.out'
  DEN='den.out'

# Programs to execute (very time consuming!)
  modfd:   'rm-fd.h' /          # gridded elastic parameters
  fd2d:    'rm-fd.h' /          # FD synthetic seismograms

# FD synthetic seismograms on a larger grid to test boundary conditions
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Larger grid to test the boundary conditions:
  O1=-.345  N1=691  N3=384

# GSE file with synthetic seismograms
  SS='rm-fda.gse'

# No snapshots for this test:
  SNAP1=' '  SNAP3=' '

# Programs to execute
 #modfd:   'rm-fd.h' /        # gridded elastic parameters
 #fd2d:    'rm-fd.h' /        # FD synthetic seismograms
# Do not forget to increase default value of MRAM in ram.inc!

# Plotting the seismograms
# ~~~~~~~~~~~~~~~~~~~~~~~~
# Plotting ray-matrix seismograms alone
  SS='rm-ss.gse'  SP1='rm-ss1.ps'  SP2=' '  SP3='rm-ss3.ps'
 #sp:      'rm-fd.h' /        # seismogram plotting

# Amplitude power scaling roughly converting line source to point source
  SPOWER=-.5

# Plotting FD seismograms alone
  SS='rm-fd.gse'  SP1='rm-fd1.ps'  SP2=' '  SP3='rm-fd3.ps'
 #sp:      'rm-fd.h' /        # seismogram plotting

# Comparison of the seismograms
  SS=' '
  SS='rm-fd.gse'   KOLOR=4   SPOWER=-.5   # FD seismograms (blue)
# SS1='rm-fda.gse' KOLOR1=3  SPOWER1=-.5  # test FD seismograms (green)
  SS2='rm-ss.gse'  KOLOR2=2  SPOWER2=0.0  # ray-matrix seismograms (red)
  SP1='rm-1.ps'  SP2=' '  SP3='rm-3.ps'
  sp:      'rm-fd.h' /        # seismogram plotting

